% Ten Tusscher et al model
% Am J Physiol 286: H1573H1589, 2004.
% human ventricular cell 
% improved version of Priebe-Beucklemann
%    
%    t   time variable
%    V   membrane potantial
clear all
load tt_fixed_params
colors = repmat('krgbmc',1,500) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global F R T RTF ;
% Physical constants
F = 96.4867 ;                % Faraday constant, coulombs/mmol
R = 8.314 ;                  % gas constant, J/(K mol)
T = 273+37 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Cm S ;
Cm = 1 ;                      % uF/cm2
S = 0.2 ;                     % surface area to volume ratio

global Vmyo VSR ;
Vmyo = 16.404 ;               % pL
VSR = 1.094 ;                 % pL

global Ko Nao Cao ;
Ko = 8000 ;                   % uM
Nao = 140000 ;                % uM
Cao = 2000 ;                  % uM

global GNa_ PCa_ GK1_ GKr_ GKs_ Gto_ ;
global GpK_ GpCa_ GNab_ GCab_ pKNa ;
GNa_ = 14.838 ;               % nS/pF
PCa_ = 1.75e-4 ;              % cm3 uF-1 s-1 
GK1_ = 5.405 ;                % nS/pF
GKr_ = 0.096 ;                % nS/pF

GpK_ = 0.0146 ;               % nS/pF
GpCa_ = 0.025 ;               % nS/pF
GNab_ = 2.9e-4 ;              % nS/pF
GCab_ = 5.92e-4 ;             % nS/pF

% % % Epicardial cell
Gto_ = 0.294 ;                % nS/pF
GKs_ = 0.245 ;                % nS/pF

% % % % M cell
% Gto_ = 0.294 ;                % nS/pF
% GKs_ = 0.062 ;                % nS/pF
% 
% % % % Endocardial cell
% Gto_ = 0.073 ;                % nS/pF
% GKs_ = 0.245 ;                % nS/pF
% 

% % % % Epicardial cell, M cell
% Gto_ = 0.294 ;                 % nS/pF
% % % % Endocardial cell
% Gto_ = 0.073 ;                 % nS/pF
% 
% % % % Epicardial cell, endocardial cell
% GKs_ = 0.245 ;                 % nS/pF
% % % % M cell
% GKs_ = 0.062 ;                 % nS/pF

pKNa = 0.03 ;                  % relative permeability, Na to K

global INaK_ kNaCa Iup_ Vleak
% Maximum rates of intracellular transport mechanisms
INaK_ = 1.362 ;                % pA/pF
kNaCa = 1000 ;                 % pA/pF
Iup_ = 0.425 ;                 % uM/ms
Vleak = 8e-5 ;                 % ms-1

global KmNa KmCa ksat alpha_ncx eta KmNai KmKo KpCa Kmup ;
% global eta ksat KmNa KmCa KmfCa KmK1 KmNai KmKo KmpCa Kmup  
% global KmNa KmCa KmNai KmKo sigma ;
% % values needed for calculation of NCX current
% Several saturation constants, nomenclature gets tricky
% NCX
KmNa = 87500 ;                 % uM
KmCa = 1380 ;                  % uM
ksat = 0.1 ;                   % unitless
alpha_ncx = 2.5 ;              % unitless
eta = 0.35 ;                   % unitless, actually gamma in paper
% Na-K pump
KmNai = 40000 ;                % uM
KmKo = 1000 ;                  % uM
% Sarcolemmal Ca pump
KpCa = 0.5 ;                    % uM
% SERCA
Kmup = 0.25 ;                  % uM

global arel brel crel ;
arel = 16.464 ;                 % uM/ms
brel = 250 ;                   % uM
crel = 8.232 ;                  % uM/ms

% % % Buffering parameters
% % % Considered generic cytotolic and SR buffers
global Bufc Kbufc BufSR KbufSR 
Bufc = 150 ;                % uM
Kbufc = 1 ;                   % uM
BufSR = 10000 ;              % uM
KbufSR = 300 ;                 % uM 

baselineparameters = [GNa_;GNab_;PCa_;GCab_;Gto_;GKr_;GKs_;GK1_;GpK_; ...
  INaK_;kNaCa;arel;crel;Vleak;Iup_;GpCa_] ;

n_parameters = length(baselineparameters) ;
variations = 300 ;
% variations = 1 ;
sigma = 0.1 ;     % standard deviation of variation of parameters
% sigma = 0 ;

%  stimints = 370:-5:100 ;
%  %stimints = 400 ;
%  pacingrates = length(stimints) ;
%  allparameters = zeros(n_parameters,variations) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 2:  Define simulation, stimulus, and recording parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stiminterval = 1000 ;      % interval bewteen stimuli, ms
stimdelay = 50 ;
stimdur = 2 ;
stim_amp = 20 ;

tend = 80*stiminterval ;              % end of simulation, ms

n_stimuli = ceil((tend-stimdelay)/stiminterval) ;
stim_starts = stimdelay + stiminterval*(0:n_stimuli-1)  ;
stim_ends = stim_starts + stimdur ;

simints = 2*n_stimuli+1 ;
intervals(1,:) = [0,stim_starts(1)] ;
for i=1:n_stimuli-1
  intervals(2*i,:) = [stim_starts(i),stim_ends(i)] ;
  intervals(2*i+1,:) = [stim_ends(i),stim_starts(i+1)] ;
end
intervals(2*n_stimuli,:) = [stim_starts(n_stimuli),stim_ends(n_stimuli)] ;
intervals(2*n_stimuli+1,:) = [stim_ends(n_stimuli),tend] ;

Istim = zeros(simints,1) ;
stimindices = 2*(1:n_stimuli) ;
Istim(stimindices) = -stim_amp ;

% figure(1)
% clf
% hold on

for ii=1:variations

  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
  [GNa_,GNab_,PCa_,GCab_,Gto_,GKr_,GKs_,GK1_,GpK_, ...
    INaK_,kNaCa,arel,crel,Vleak,Iup_,GpCa_] = deal(newparamscell{:}) ;
  
V = -85 ;
m = 0 ;
h = 1 ;
j = 1 ;
d = 0 ;
f = 1 ;
fCa = 1 ;
r = 0 ;
s = 1 ;
xs = 0 ;
xr1 = 1 ;
xr2 = 1 ;
g = 0 ;
Cai = 0.1 ;
CaSR = 1000 ;
Nai = 10000 ;
Ki = 130000 ;

statevar_i = [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] ;


%% Let rest for 120 seconds without any stimuli applied 
  [post,posstatevars] = ode15s(@dydt_tentusscher,[0,30000],statevar_i,[],0) ;
  statevar_i = posstatevars(end,:) ;

t = 0 ;
statevars = statevar_i ;
for i=1:simints
    [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
 
  t = [t;post(2:end)] ;
  statevars = [statevars;posstatevars(2:end,:)] ;
  statevar_i = posstatevars(end,:) ;
end

outputcell = num2cell(statevars,1) ;
nvariables = length(outputcell) ;

[V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;
% figure(1)
% hold on
% plot(t,V,colors(mod(ii,length(colors))+1))

  actionpotentials{2*ii-1} = t ;
  actionpotentials{2*ii} = V ;
  allparameters(:,ii) = newparams' ;

% tregular = 0:0.1:tend ;
% Vregular = interp1(t,V,tregular) ;
% t = tregular ;
% V = Vregular ;

%% First Derivative 
t_diff = diff(t);
V_diff = diff(V);
deriv = V_diff./t_diff;
[dVdtmax,dVdtmax_dex] = max(deriv) ;
guyer = (t+t([2:end,end]))./2;
t_deriv = guyer(1:end-1);
%% Second Derivative

t_diff2 = diff(t_deriv);
V_diff2 = diff(deriv);
deriv_2 = V_diff2./t_diff2;
guyer_2 = (t_deriv + t_deriv([2:end,end]))./2;
t_deriv2 = guyer_2(1:end-1);

% x1 = t_deriv(1:300);
% y1 = deriv(1:300);
% x2 = t_deriv2(1:300);
% y2 = deriv_2(1:300);
x1 = t_deriv;
y1 = deriv;
x2 = t_deriv2;
y2 = deriv_2;

%% Local Max
%   Find local maxima in vector XX,where
%	LMVAL is the output vector with maxima values, INDD  is the 
%	corresponding indices, FILT is the number of passes of the small
%	running average filter in order to get rid of small peaks.  Default
%	value FILT =0 (no filtering). FILT in the range from 1 to 3 is 
%	usially sufficient to remove most of a small peaks
	
x = deriv;
x = V ;
len_x = length(x);
	fltr=[1 1 1]/3;
  if nargin <2, filt=0; 
	else
x1=x(1); x2=x(len_x); 
	for jj=1:filt,
	c=conv(fltr,x);
	x=c(2:len_x+1);
	x(1)=x1;  
        x(len_x)=x2; 
	end
  end
lmval=[]; indd=[];
i=2;		% start at second data point in time series
    while i < len_x-1,
	if x(i) > x(i-1)
	   if x(i) > x(i+1)	% definite max
lmval =[lmval x(i)];
indd = [ indd i];
	   elseif x(i)==x(i+1)&x(i)==x(i+2)	% 'long' flat spot
%lmval =[lmval x(i)];  	%1   comment these two lines for strict case
%indd = [ indd i];	%2 when only  definite max included
i = i + 2;  		% skip 2 points
	   elseif x(i)==x(i+1)	% 'short' flat spot
%lmval =[lmval x(i)];	%1   comment these two lines for strict case
%indd = [ indd i];	%2 when only  definite max included
i = i + 1;		% skip one point
	   end
	end
	i = i + 1;
    end
if filt>0 & ~isempty(indd),
	if (indd(1)<= 3)|(indd(length(indd))+2>length(xx)), 
	   rng=1;	%check if index too close to the edge
	else rng=2;
	end
	  for iii=1:length(indd), 	% Find the real maximum value
	    [val(iii) iind(iii)] = max(xx(indd(iii) -rng:indd(iii) +rng));
	    iind(iii)=indd(iii) + iind(iii)  -rng-1;
	  end
  indd=iind; lmval=val;
else
end

indices = find(indd > dVdtmax_dex) ;
indd = indd(indices) ;

%% Local Min
%   Find local minima in vector X, where LMVAL is the output
%	vector with minima values, INDD is the corresponding indices 
%	FILT is the number of passes of the small running average filter
%	in order to get rid of small peaks.  Default value FILT =0 (no
%	filtering). FILT in the range from 1 to 3 is usially sufficient to 
%	remove most of a small peaks

a=deriv;
a = V ;
len_a = length(a);
	fltr=[1 1 1]/3;
  if nargin <2, filt=0; 
	else
x1=a(1); x2=a(len_a); 

	for jj=1:filt,
	c=conv(fltr,a);
	a=c(2:len_a+1);
	a(1)=x1;  
        a(len_a)=x2; 
	end
  end

lmval_a=[];
indd_a=[];
i=2;		% start at second data point in time series

    while i < len_a-1,
	if a(i) < a(i-1)
	   if a(i) < a(i+1)	% definite min
lmval_a =[lmval_a a(i)];
indd_a = [ indd_a i];

	   elseif a(i)==a(i+1)&a(i)==a(i+2)	% 'long' flat spot
%lmval =[lmval x(i)];	%1   comment these two lines for strict case 
%indd = [ indd i];	%2 when only  definite min included
i = i + 2;  		% skip 2 points

	   elseif a(i)==a(i+1)	% 'short' flat spot
%lmval =[lmval x(i)];	%1   comment these two lines for strict case
%indd = [ indd i];	%2 when only  definite min included
i = i + 1;		% skip one point
	   end
	end
	i = i + 1;
    end

if filt>0 & ~isempty(indd_a),
	if (indd_a(1)<= 3)|(indd_a(length(indd_a))+2>length(a)), 
	   rng=1;	%check if index too close to the edge
	else rng=2;
	end

	   for iii=1:length(indd_a), 
		[val(iii) iind_a(iii)] = min(a(indd_a(iii) -rng:indd_a(iii) +rng));
		iind_a(iii)=indd_a(iii) + iind_a(iii)  -rng-1;
	   end
  indd_a=iind_a; lmval_a=val;
else
end

indices = find(indd_a > dVdtmax_dex) ;
indd_a = indd_a(indices) ;

Vmax_min_hyper(ii)=V(indd(1))-V(indd_a(1));
Vmin_max_hyper(ii)=V(indd(2))-V(indd_a(1));
%tmax_min(ii)=t(indd(1))-t(indd_a(1));
tmin_max_hyper(ii)=t(indd(2))-t(indd_a(1));

if (mod(ii,50) == 0)
  disp(['Completed ',int2str(ii),' iterations'])
end

end

save derivs_fixed_hyper allparameters actionpotentials variations n_parameters Vmax_min_hyper Vmin_max_hyper tmin_max_hyper